#ifndef BPIPEINPUT_H
#define BPIPEINPUT_H

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of mergerays
//
// mergerays is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// mergerays is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <string>
#include <vector>
#include <list>
#include <memory>

#include <bpipe/bpipe.h>
#include <bpipe/bpipe_axaf.h>

#include "DataField.h"

//
//: A Bpipe Input class to open a list of input bpipe.
//  The class is responsible for tranferring all header fields
//  to the output stream unless they are requested to be transferred to
//  the data field.
//
class BpipeInput {

    friend ostream& operator << ( ostream& os, BpipeInput& a ) {
        a.print( os );
        return os;
    }

    friend ostream& operator << ( ostream& os, BpipeInput* a ) {
        os << *a;
        return os;
    }

public:

    virtual ~BpipeInput( );

    BpipeInput( string& fnames, const std::vector<std::string>& xfer_fields );

    size_t get_n_dpkts( size_t i ) { return n_dpkts.at(i); }

    size_t get_n_dpkts_sum( ) { return n_dpkts_sum; }

    // return the number of input streams.
    size_t get_num( ) { return bpipes.size(); }

    // return the ith input stream.
    BPipe* get_bpipe( size_t i ) {
        return bpipes.at(i);
    }

    bool have_valid_npkts() { return get_num() == n_dpkts_valid; }

    void print( ostream& os ) { os << ""; }

    void resize_core( std::list<DataFieldPtr>& data_fields, const std::vector<std::string>& xfer_fields );

    //: Read from the ith input stream.
    size_t read_dpkts( size_t i );

protected:

    //: The input bpipe streams
    std::vector<BPipe*> bpipes;

    std::vector<void*>  data;

    //: The number of input streams containing the the n_pkts field.
    size_t       n_dpkts_valid = 0;

    //: the number of data packet in each stream.
    std::vector<size_t>  n_dpkts;

    //: The total number of data packets in all of the streams
    size_t n_dpkts_sum;

private:

    void attach_bpipes( vector<string>& filenames );
    void header_to_datafield( const std::vector<std::string>& xfer_fields );

    //: declare but (purposedly) do not define
    BpipeInput( const BpipeInput& tmp );
    BpipeInput& operator = ( const BpipeInput& rhs );

};


#endif
