#ifndef BPIPEOUTPUT_H
#define BPIPEOUTPUT_H

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of mergerays
//
// mergerays is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// mergerays is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <vector>
#include <list>

#include <bpipe/bpipe.h>
#include <bpipe/bpipe_axaf.h>

#include <cxcparamx/simple.h>

#include "BpipeInput.h"
#include "DataField.h"

//
//: The base class for the Bpipe Output.  The class is responsible for
//: determining the common output data field.
//  The class loops over the input streams to determine the data fields
//  which have the same name, data type and the greatest common denominator
//  matrix size for all the data fields.
//
class BpipeOutput {

public:

    virtual ~BpipeOutput( );

    BpipeOutput( BpipeInput& in, const std::string& output, const std::vector<std::string>& xfer_fields );

    virtual void write_dpkts();

  
protected:
  

    BPipe*       bpipe = NULL;
    BPipeOutput* bpo = NULL;
    void*        data = NULL;

    std::vector<DataFieldPtr> output_data_fields;
    //: The output data fields.

    const std::string output;
    //: The output filename

    BpipeInput& input;
    //: An association to the input streams.

    virtual size_t copy_data( size_t i );
    //: loop through the output_data_fields array to copy
    //: the data for the datafields from the ith stream.

private:

    void add_header_fields( BPipe* input, const std::vector<std::string>& xfer_fields,
                            std::list<DataFieldPtr>& data_fields );
    //: Get the header from the input bpipe, make sure that the fields
    //: specified by the xfer parameter are not written to the output header.

    void get_data_field_ptr( std::list<DataFieldPtr>& data_fields );

    void init_bpipe_output( std::list<DataFieldPtr>& data_fields );

    void select_data_fields( BPipe* bpipe_input,
                             const std::vector<string>& xfer_fields,
                             std::list<DataFieldPtr>& data_fields );

    void select_data_fields( const char* n, BPDataType t, BPMatrix* matrix, std::list<DataFieldPtr>& data_fields );

    //: Transfer the output data fields from a linklist to an array of
    //: DataField to speed things up.
    void set_output_data_fields( std::list<DataFieldPtr>& data_fields );
};

#endif
