// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of mergerays
//
// mergerays is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// mergerays is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <string>
#include <limits>
#include <float.h>
#include <cstdlib>

#include <bpipe/bpipe.h>
#include <bpipe/bpipe_axaf.h>

#include <tracefctxx/TraceFct.h>

#include "DataField.h"


//: copy the data from the ith stream for the `name' datafield.
void
DataField::copy_data( int i ) {
    bpipe_data_copy( output_data, input_data[ i ], matrix, type );
}

size_t
DataField::min_time( std::vector<bool>& empty_stream ) {

    size_t bin = 0;
    double mintime = DBL_MAX;

    for ( size_t i = 0; i < count; i++ ) {
        if ( ! empty_stream[ i ] ) {
            double time_i = *( (double*) input_data[ i ] );
            if ( time_i < mintime ) {
                mintime = time_i;
                bin = i;
            }
        }
    }

    return bin;
}
//: Find which stream has the minimum time.
//: It is up to the user to make sure that 
//: the name of the data field is `time'

void
DataField::update( DataField* datafield ) { 

    if ( name != datafield->get_name( )
         &&
	 type != datafield->get_type( ) )
        return;
    // If the name and the type do not match then don't update.

    count++;

    BPMatrix* this_matrix = matrix;

    BPMatrix* datafield_matrix = datafield->get_matrix( );

    matrix = bpipe_matrix_min( this_matrix, datafield_matrix );
    if ( NULL == matrix )
        TraceFct( __func__ ).die( "Unable to allocate matrix matrix" );

    bpipe_matrix_delete( this_matrix );
}
