#ifndef DATAFIELD_H
#define DATAFIELD_H

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of mergerays
//
// mergerays is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// mergerays is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <memory>
#include <string>
#include <vector>

#include <bpipe/bpipe.h>
#include <bpipe/bpipe_axaf.h>

class DataField {


public:

    DataField( const char* _name, BPDataType t, BPMatrix* mat, int nbpipe )
        : count( 1 )
        , type( t )
        , name( _name )
        , matrix( mat )
        , input_data( nbpipe, NULL )
    {
    }

    void copy_data( int i );
    //: copy the data from the ith stream for the `name' datafield.

    size_t     get_count( ) { return count; }

    BPMatrix*  get_matrix( ) { return matrix; }

    const std::string& get_name( ) const { return name; }

    void*      get_input_data( int i ) { return input_data[ i ]; }

    BPDataType get_type( ) { return type; }

    void       set_input_data( int i, void* ptr ) { input_data[ i ] = ptr; }

    void       set_output_data( void* ptr ) { output_data = ptr; }

    //: Find which stream has the minimum time.
    //: It is up to the user to make sure that 
    //: the name of the data field is `time'
    size_t min_time( std::vector<bool>& empty_stream );

    void update( DataField* datafield );

protected:

    size_t     count;
    //: The number of streams containing data field with the 
    //: following: type, name, and matrix properties.

    BPDataType type;
    //: The datatype for the data field.

    std::string       name;
    //: The name of the data field.

    BPMatrix*  matrix = NULL;
    //: The dimension of the `name' data field.

    std::vector<void*>     input_data;
    //: The pointer to the input data for the datafield `name'.
    //  This is an array of pointers since the pointer must are
    //  different for the various input streams.

    void*      output_data = NULL;
    //: The pointer to the output data.

    void*      header_data = NULL;
    //: The pointer to the header data.

};

using DataFieldPtr = std::unique_ptr<DataField>;


#endif
