// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of mergerays
//
// mergerays is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// mergerays is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <vector>
#include <algorithm>

#include <tracefctxx/TraceFct.h>
#include <randlib_c/randlib.h>

#include "SampleBpipeOutput.h"

SampleBpipeOutput::SampleBpipeOutput( BpipeInput& input, const std::string& output,
                                      const std::vector<std::string>& xfer_fields,
                                      long block, long seed1, long seed2
                                      ) :
    BpipeOutput( input, output, xfer_fields ) {

    // All the input streams must have the flag
    // BPHN_n_dpkts_valid set to a non-zero value.
    if ( ! input.have_valid_npkts( ) )
        TraceFct(__func__).die( "Not all the input streams have valid numbers of data packets " );

    /* initialize random number generator and get to correct block */
    setall( seed1, seed2 );
    for ( long n = 0 ; n < block ; n++ )
        initgn(1);
}

void
SampleBpipeOutput::write_dpkts() {

    size_t ndpkts = input.get_n_dpkts_sum();

    std::vector<double> distribution( input.get_num() );

    double last = 0;
    size_t j = 0;
    for ( auto& cdf : distribution ) {
        cdf = last + input.get_n_dpkts( j++ ) / double( ndpkts );
        last = cdf;
    }

    auto begin = distribution.begin();
    auto end   = distribution.end();
    while ( ndpkts ) {
        double random = ranf( );

        auto stream_num = std::find_if( begin, end,
                                        [random](double cdf) { return random < cdf; }
                                        ) - begin;

        if ( 0 == copy_data( stream_num ) )
            continue;

        for( auto& datafield : output_data_fields )
            datafield->copy_data( stream_num );
    
        if ( bpipe_write_dpkt( bpipe, data, bpo ) )
            TraceFct(__func__).die( std::string("error writing data packet: ") + bpipe_strerror( bpipe_errno ) );

        --ndpkts;
    }

}
