// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of mergerays
//
// mergerays is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// mergerays is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <tracefctxx/TraceFct.h>

#include "TimeBpipeOutput.h"

TimeBpipeOutput::TimeBpipeOutput( BpipeInput& input, const std::string& output,
                                  const std::vector<std::string>& xfer_fields)
    : BpipeOutput( input, output, xfer_fields )
    , num( input.get_num() )
    , time_datafield( NULL )
{

    for ( auto& datafield : output_data_fields )
        if ( datafield->get_name() == BP_AXAF_time ) {
            time_datafield = datafield.get();
            break;
        }

    if ( ! time_datafield )
        TraceFct(__func__).die( std::string("The field `" )
                                + BP_AXAF_time + "' is not present in all of the input streams" );
}

size_t TimeBpipeOutput::copy_data( size_t i ) {

    return input.read_dpkts( i );
}

void
TimeBpipeOutput::write_dpkts(  ) {

    size_t num_empty_stream = 0;

    std::vector<bool> empty_stream;

    for ( size_t i = 0; i < num; i++ ) {
        if ( 0 == copy_data( i ) ) {
            empty_stream.push_back( true );
            ++num_empty_stream;
        } else {
            empty_stream.push_back( false );
        }
    }

    while( num_empty_stream < num ) {

        size_t stream_num = time_datafield->min_time( empty_stream );

        for( auto& datafield : output_data_fields )
            datafield->copy_data( stream_num );
        bpipe_write_dpkt( bpipe, data, bpo );

        // read in next data packet from this stream
        if ( 0 == copy_data( stream_num ) ) {
            empty_stream[ stream_num ] = true;
            ++num_empty_stream;
        }
    }
}
