// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of mergerays
//
// mergerays is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// mergerays is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>

#include <suplibxx/str.h>
#include <tracefctxx/TraceFct.h>
#include <cxcparamx/simple.h>

#include "BpipeInput.h"
#include "TimeBpipeOutput.h"
#include "SampleBpipeOutput.h"

#include "config.h"

void mergeray( cxcparamx::simple& parfile ) {

    TraceFct tf( "mergeray" );

    std::string xfer( parfile.getstr( "xfer" ) );
    std::string filenames( parfile.getstr( "input" ) );
    std::string output( parfile.getstr( "output" ) );
    std::string merge_method( parfile.getstr( "method" ) );

    std::vector<std::string> xfer_fields;
    suplib::tok( xfer_fields, xfer, "," );

    BpipeInput input( filenames, xfer_fields);


    if      ( "cat" == merge_method )
        BpipeOutput( input, output, xfer_fields ).write_dpkts();

    else if ( "time" == merge_method )
        TimeBpipeOutput( input, output, xfer_fields ).write_dpkts();

    else if ( "sample" == merge_method )
        SampleBpipeOutput( input, output, xfer_fields,
                           parfile.getl( "block" ),
                           parfile.getl( "seed1" ),
                           parfile.getl( "seed2" )
                           ).write_dpkts();


}

int get_help_version( char*, cxcparamx::simple&, bool&, bool& );
int usage();

int main( int argc, char** argv ) {

    TraceFct tf( argv[0], 0, -1 );

    try {

        cxcparamx::simple parfile( argc, argv );

        if ( parfile.getb( "help" ) )
            usage();

        else if ( parfile.getb( "version" ) )
            cout << argv[0] << "\t" << VERSION << endl;

        else 
            mergeray( parfile );

    } catch ( Exception& E ) {

        tf.exit( EXIT_FAILURE, E );

    } catch ( exception& e ) { 

        tf.exit( EXIT_FAILURE, e.what() );

    }
    return EXIT_SUCCESS;
}

