#include <string>

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>
#include <bpipe/bpipe_axaf.h>

/* simple print error and exit routine */
#define error(string)                                                   \
  do {                                                                  \
      fprintf( stderr, __FILE__ " %d: %s\n", __LINE__, string );        \
      exit( EXIT_FAILURE );                                             \
     } while (0)

/*
  invoke as

  create

*/


/*

    merge on time
    merge on sample (requires n_dpkts_valid and n_dpkts in header)
    concatentate
    move header field to data packets.

    create <mode> <file id> <id start> <time start>
    mode = comma separated list

    * sample
    
    add n_dpkts_valid and n_dpkts

      * header

 */


int 
main (int argc, char *argv[]) {

  if ( argc != 5 )
      error( "usage: create stream_id[int] mode[sample,header,time]  id_start[int] time_start[float]" );

  size_t idx = 0;
  int stream_id = std::stoi( argv[++idx] );
  std::string mode = "," + std::string(argv[++idx]) + ",";
  int id_start = std::stoi( argv[++idx] );
  double time_start = std::stod( argv[++idx] );

  auto npos = std::string::npos;
  bool mode_sample = npos != mode.find( ",sample," );
  bool mode_header = npos != mode.find( ",header," );
  bool mode_time   = npos != mode.find( ",time," );

  unsigned int n_dpkts = 10;
  int          n_dpkts_valid = 1;

  BPipe *bpipe( bpipe_new() );
  if ( ! bpipe )
      error( bpipe_strerror( bpipe_errno ) );

  size_t i;


  /* create new data packet fields */

  if ( bpipe_dpktf_add( bpipe,  BP_AXAF_phot_id,     BPDType_uint,   NULL ) )
    error( bpipe_strerror( bpipe_errno ) );

  if ( mode_time
      &&  bpipe_dpktf_add( bpipe,  BP_AXAF_time,     BPDType_double,   NULL ) )
      error( bpipe_strerror( bpipe_errno ) );

  if ( mode_header )
      bpipe_hdrf_add( bpipe, "hdrfield", BPDType_int, NULL, &stream_id, true );

  if ( mode_sample ) {
      bpipe_hdrf_add( bpipe, BPHN_n_dpkts, BPHT_n_dpkts, NULL, &n_dpkts, false );
      bpipe_hdrf_add( bpipe, BPHN_n_dpkts_valid, BPHT_n_dpkts_valid, NULL, &n_dpkts_valid, false );
  }

  BPipeOutput *bpo ( bpipe_output( bpipe, "stdout" ) );
  if ( ! bpo )
    error( bpipe_strerror( bpipe_errno ) );
    
  void *data ( bpipe_map_alloc( bpipe, 1, NULL ) );
  if ( ! data )
    error( bpipe_strerror( bpipe_errno ) );


  /* output header */
  if ( bpipe_write_hdr(bpipe) )
    error( bpipe_strerror( bpipe_errno ) );

  /* get handles to data packet fields */
  unsigned int *id_p = static_cast<unsigned int*> (bpipe_dpktf_datap( bpipe, data, BP_AXAF_phot_id, BPDType_uint ) );

  double time;
  double *time_p =  mode_time
      ? static_cast<double*>( bpipe_dpktf_datap( bpipe, data, BP_AXAF_time, BPDType_double ) )
      : &time;

  *time_p = time_start;
  *id_p = id_start;

  for (i = 0; i < n_dpkts ; i++) {
    
    if ( bpipe_write_dpkt( bpipe, data, bpo ) )
      error( "error writing photon" );

    (*id_p)++;
    (*time_p)++;

  }

  bpipe_delete(bpipe);

  return EXIT_SUCCESS;
}

